@@ -345,8 +345,8 @@ svg_plugin_set_window (NPP instance, NPWindow * window)
 }
 
 static NPError
-svg_plugin_new_stream (NPP instance, NPMIMEType type,
-					   const char *window, NPStream ** stream_ptr)
+svg_plugin_new_stream (NPP instance, NPMIMEType type, NPStream *stream,
+		      NPBool seekable, uint16 *stype)
 {
 	SvgPlugin *plugin;
 
@@ -362,6 +362,8 @@ svg_plugin_new_stream (NPP instance, NPMIMEType type,
 	g_return_val_if_fail(plugin->bytes == NULL, NPERR_NO_ERROR);
 
 	plugin->bytes = g_byte_array_new();
+	
+	*stype = NP_NORMAL;
 
 	return NPERR_NO_ERROR;
 }
@@ -381,11 +383,13 @@ svg_plugin_destroy_stream (NPP instance, NPStream * stream, NPError reason)
 	plugin = (SvgPlugin *) instance->pdata;
 	if (plugin == NULL)
 		return NPERR_NO_ERROR;
-	
-	url_len = strlen(stream->url);
-	plugin->base_url = mozilla_funcs.memalloc (url_len + 1);
-	strcpy(plugin->base_url, stream->url);
-	plugin->base_url[url_len] = '\0';
+
+	if (stream->url) {
+		url_len = strlen(stream->url);
+		plugin->base_url = mozilla_funcs.memalloc (url_len + 1);
+		strcpy(plugin->base_url, stream->url);
+		plugin->base_url[url_len] = '\0';
+	}
 
 	/* trigger */
 	res = svg_plugin_redraw (plugin);
