@@ -42,6 +42,9 @@
 #include "rsvg-filter.h"
 #include "rsvg-mask.h"
 #include "rsvg-marker.h"
+#include "rsvg-cairo-render.h"
+
+#include <libxml/uri.h>
 
 #include <math.h>
 #include <string.h>
@@ -796,6 +799,36 @@ rsvg_entity_decl (void *data, const xmlChar * name, int type,
     RsvgHandle *ctx = (RsvgHandle *) data;
     GHashTable *entities = ctx->priv->entities;
     xmlEntityPtr entity;
+#if LIBXML_VERSION >= 20700
+    xmlChar *resolvedSystemId = NULL, *resolvedPublicId = NULL;
+
+    if (systemId)
+        resolvedSystemId = xmlBuildRelativeURI (systemId, (xmlChar*) rsvg_handle_get_base_uri (ctx));
+    else if (publicId)
+        resolvedPublicId = xmlBuildRelativeURI (publicId, (xmlChar*) rsvg_handle_get_base_uri (ctx));
+
+    if (type == XML_EXTERNAL_PARAMETER_ENTITY && !content) {
+      GByteArray *arr;
+
+      if (systemId)
+        arr = _rsvg_acquire_xlink_href_resource ((const char *) systemId,
+                                                 rsvg_handle_get_base_uri (ctx), NULL);
+      else if (publicId)
+        arr = _rsvg_acquire_xlink_href_resource ((const char *) publicId,
+                                                 rsvg_handle_get_base_uri (ctx), NULL);
+      if (arr) {
+        content = xmlCharStrndup ((const char*)arr->data, arr->len);
+        g_byte_array_free(arr, TRUE);
+      }
+    }
+      
+    entity = xmlNewEntity(NULL, name, type, resolvedPublicId, resolvedSystemId, content);
+
+    free(resolvedPublicId);
+    free(resolvedSystemId);
+
+    g_hash_table_insert (entities, g_strdup ((const char*) name), entity);
+#else
     xmlChar *dupname;
 
     entity = xmlMalloc (sizeof (xmlEntity));
@@ -834,6 +867,7 @@ rsvg_entity_decl (void *data, const xmlChar * name, int type,
     }
 
     g_hash_table_insert (entities, dupname, entity);
+#endif
 }
 
 static void
@@ -845,6 +879,17 @@ rsvg_unparsed_entity_decl (void *ctx,
     rsvg_entity_decl (ctx, name, XML_INTERNAL_GENERAL_ENTITY, publicId, systemId, NULL);
 }
 
+static xmlEntityPtr
+rsvg_get_parameter_entity (void *data, const xmlChar * name)
+{
+    RsvgHandle *ctx = (RsvgHandle *) data;
+    xmlEntityPtr entity;
+
+    entity = g_hash_table_lookup (ctx->priv->entities, name);
+
+    return entity;
+}
+
 #endif
 
 static void
@@ -913,6 +958,7 @@ rsvg_SAX_handler_struct_init (void)
         rsvgSAXHandlerStruct.getEntity = rsvg_get_entity;
         rsvgSAXHandlerStruct.entityDecl = rsvg_entity_decl;
         rsvgSAXHandlerStruct.unparsedEntityDecl = rsvg_unparsed_entity_decl;
+        rsvgSAXHandlerStruct.getParameterEntity = rsvg_get_parameter_entity;
 #endif
         rsvgSAXHandlerStruct.characters = rsvg_characters;
         rsvgSAXHandlerStruct.error = rsvg_error_cb;
@@ -1039,6 +1085,24 @@ rsvg_error_quark (void)
     return g_quark_from_string ("rsvg-error-quark");
 }
 
+static void
+rsvg_set_error (GError **error, xmlParserCtxtPtr ctxt)
+{
+  xmlErrorPtr xerr;
+
+  xerr = xmlCtxtGetLastError (ctxt);
+  if (xerr) {
+    g_set_error (error, rsvg_error_quark (), 0,
+                 _("Error domain %d code %d on line %d column %d of %s: %s"),
+                 xerr->domain, xerr->code,
+                 xerr->line, xerr->int2,
+                 xerr->file ? xerr->file : "data",
+                 xerr->message ? xerr->message: "-");
+  } else {
+    g_set_error (error, rsvg_error_quark (), 0, _("Error parsing XML data"));
+  }
+}
+
 static gboolean
 rsvg_handle_write_impl (RsvgHandle * handle, const guchar * buf, gsize count, GError ** error)
 {
@@ -1060,7 +1124,7 @@ rsvg_handle_write_impl (RsvgHandle * handle, const guchar * buf, gsize count, GE
 
     result = xmlParseChunk (handle->priv->ctxt, (char *) buf, count, 0);
     if (result != 0) {
-        g_set_error (error, rsvg_error_quark (), 0, _("Error parsing XML data"));
+        rsvg_set_error (error, handle->priv->ctxt);
         return FALSE;
     }
 
@@ -1094,7 +1158,7 @@ rsvg_handle_close_impl (RsvgHandle * handle, GError ** error)
         xmlFreeDoc (xmlDoc);
 
         if (result != 0) {
-            g_set_error (error, rsvg_error_quark (), 0, _("Error parsing XML data"));
+            rsvg_set_error (error, handle->priv->ctxt);
             return FALSE;
         }
     }
@@ -1114,9 +1178,8 @@ rsvg_handle_close_impl (RsvgHandle * handle, GError ** error)
 static void
 rsvg_state_free_func (gpointer data, gpointer user_data)
 {
-    RsvgDrawingCtx *ctx = (RsvgDrawingCtx *) user_data;
     rsvg_state_finalize ((RsvgState *) data);
-    g_mem_chunk_free (ctx->state_allocator, data);
+    g_slice_free (RsvgState, data);
 }
 
 void
@@ -1130,11 +1193,11 @@ rsvg_drawing_ctx_free (RsvgDrawingCtx * handle)
 	/* the drawsub stack's nodes are owned by the ->defs */
 	g_slist_free (handle->drawsub_stack);
 
+    g_slist_free (handle->ptrs);
+	
     if (handle->base_uri)
         g_free (handle->base_uri);
 
-    g_mem_chunk_destroy (handle->state_allocator);
-
     if (handle->pango_context != NULL)
         g_object_unref (handle->pango_context);
 
@@ -1207,133 +1270,6 @@ rsvg_handle_get_desc (RsvgHandle * handle)
         return NULL;
 }
 
-typedef struct {
-    RsvgRender super;
-    RsvgBbox bbox;
-} RsvgBboxRender;
-
-static void
-rsvg_bbox_render_path (RsvgDrawingCtx * ctx, const RsvgBpathDef * bpath_def)
-{
-    RsvgState *state = rsvg_state_current (ctx);
-    RsvgBpath *bpath;
-    RsvgBboxRender *render = (RsvgBboxRender *) ctx->render;
-    RsvgBbox bbox;
-    int i;
-
-    rsvg_bbox_init (&bbox, state->affine);
-    bbox.w = bbox.h = bbox.virgin = 0;
-
-    for (i = 0; i < bpath_def->n_bpath; i++) {
-        bpath = &bpath_def->bpath[i];
-
-        switch (bpath->code) {
-        case RSVG_MOVETO:
-        case RSVG_MOVETO_OPEN:
-        case RSVG_CURVETO:
-        case RSVG_LINETO:
-            bbox.x = bpath->x3;
-            bbox.y = bpath->y3;
-            rsvg_bbox_insert (&render->bbox, &bbox);
-            break;
-        default:
-            break;
-        }
-    }
-}
-
-static void
-rsvg_bbox_render_image (RsvgDrawingCtx * ctx,
-                        const GdkPixbuf * pixbuf,
-                        double pixbuf_x, double pixbuf_y, double w, double h)
-{
-    RsvgState *state = rsvg_state_current (ctx);
-    RsvgBboxRender *render = (RsvgBboxRender *) ctx->render;
-    RsvgBbox bbox;
-
-    rsvg_bbox_init (&bbox, state->affine);
-    bbox.x = pixbuf_x;
-    bbox.y = pixbuf_y;
-    bbox.w = w;
-    bbox.h = h;
-    bbox.virgin = 0;
-
-    rsvg_bbox_insert (&render->bbox, &bbox);
-}
-
-
-static void
-rsvg_bbox_render_free (RsvgRender * self)
-{
-    g_free (self);
-}
-
-static void
-rsvg_bbox_push_discrete_layer (RsvgDrawingCtx * ctx)
-{
-}
-
-static void
-rsvg_bbox_pop_discrete_layer (RsvgDrawingCtx * ctx)
-{
-}
-
-static void
-rsvg_bbox_add_clipping_rect (RsvgDrawingCtx * ctx, double x, double y, double w, double h)
-{
-}
-
-static RsvgBboxRender *
-rsvg_bbox_render_new ()
-{
-    RsvgBboxRender *render = g_new0 (RsvgBboxRender, 1);
-    double affine[6];
-
-    render->super.free = rsvg_bbox_render_free;
-    render->super.render_image = rsvg_bbox_render_image;
-    render->super.render_path = rsvg_bbox_render_path;
-    render->super.pop_discrete_layer = rsvg_bbox_pop_discrete_layer;
-    render->super.push_discrete_layer = rsvg_bbox_push_discrete_layer;
-    render->super.add_clipping_rect = rsvg_bbox_add_clipping_rect;
-    render->super.get_image_of_node = NULL;
-    _rsvg_affine_identity (affine);
-    rsvg_bbox_init (&render->bbox, affine);
-
-    return render;
-}
-
-static RsvgBbox
-_rsvg_find_bbox (RsvgHandle * handle)
-{
-    RsvgDrawingCtx *ctx = g_new (RsvgDrawingCtx, 1);
-    RsvgBbox output;
-    RsvgBboxRender *render = rsvg_bbox_render_new ();
-    ctx->drawsub_stack = NULL;
-    ctx->render = (RsvgRender *) render;
-
-    ctx->state = NULL;
-
-    ctx->state_allocator = g_mem_chunk_create (RsvgState, 256, G_ALLOC_AND_FREE);
-
-    ctx->defs = handle->priv->defs;
-    ctx->base_uri = g_strdup (handle->priv->base_uri);
-    ctx->dpi_x = handle->priv->dpi_x;
-    ctx->dpi_y = handle->priv->dpi_y;
-    ctx->vb.w = 512;
-    ctx->vb.h = 512;
-    ctx->pango_context = NULL;
-
-    rsvg_state_push (ctx);
-    _rsvg_affine_identity (rsvg_state_current (ctx)->affine);
-    _rsvg_node_draw_children ((RsvgNode *) handle->priv->treebase, ctx, 0);
-    rsvg_state_pop (ctx);
-
-    output = render->bbox;
-    rsvg_render_free (ctx->render);
-    g_free (ctx);
-    return output;
-}
-
 /**
  * rsvg_handle_get_dimensions
  * @handle: A #RsvgHandle
@@ -1346,41 +1282,267 @@ _rsvg_find_bbox (RsvgHandle * handle)
 void
 rsvg_handle_get_dimensions (RsvgHandle * handle, RsvgDimensionData * dimension_data)
 {
-    RsvgNodeSvg *sself;
+    /* This function is probably called from the cairo_render functions.
+     * To prevent an infinite loop we are saving the state.
+     */
+    if (!handle->priv->in_loop) {
+	handle->priv->in_loop = TRUE;
+	rsvg_handle_get_dimensions_sub (handle, dimension_data, NULL);
+	handle->priv->in_loop = FALSE;
+    } else {
+	/* Called within the size function, so return a standard size */
+	dimension_data->em = dimension_data->width = 1;
+	dimension_data->ex = dimension_data->height = 1;
+    }
+}
+
+/**
+ * rsvg_handle_get_dimensions_sub
+ * @handle: A #RsvgHandle
+ * @dimension_data: A place to store the SVG's size
+ * @id: An element's id within the SVG, or NULL to get the dimension of the whole SVG. 
+ * For example, if you have a layer called "layer1" for that you want to get the dimension, 
+ * pass "#layer1" as the id.
+ *
+ * Get the size of a subelement of the SVG file. Do not call from within the size_func callback, because an infinite loop will occur.
+ *
+ * Since: 2.22
+ */
+gboolean
+rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimension_data, const char *id)
+{
+    cairo_t *cr;
+    cairo_surface_t *target;
+    RsvgDrawingCtx *draw;
+    RsvgCairoRender *render;
+    RsvgNodeSvg *root = NULL;
+    RsvgNode *sself = NULL;
     RsvgBbox bbox;
 
-    g_return_if_fail (handle);
-    g_return_if_fail (dimension_data);
+    gboolean handle_subelement = TRUE;
+
+    g_return_val_if_fail (handle, FALSE);
+    g_return_val_if_fail (dimension_data, FALSE);
 
     memset (dimension_data, 0, sizeof (RsvgDimensionData));
 
-    sself = (RsvgNodeSvg *) handle->priv->treebase;
-    if (!sself)
-        return;
+    if (id && *id) {
+	sself = (RsvgNode *) rsvg_defs_lookup (handle->priv->defs, id);
+
+	if (sself == (RsvgNode *) handle->priv->treebase)
+	    id = NULL;
+    }
+    else
+	sself = (RsvgNode *) handle->priv->treebase;
+
+    if (!sself && id)
+	return FALSE;
+
+    root = (RsvgNodeSvg *) handle->priv->treebase;
+
+    if (!root)
+	return FALSE;
 
     bbox.x = bbox.y = 0;
     bbox.w = bbox.h = 1;
 
-    if (sself->w.factor == 'p' || sself->h.factor == 'p') {
-        if (sself->vbox.active && sself->vbox.w > 0. && sself->vbox.h > 0.) {
-            bbox.w = sself->vbox.w;
-            bbox.h = sself->vbox.h;
-        } else
-            bbox = _rsvg_find_bbox (handle);
+    if (!id && (root->w.factor == 'p' || root->h.factor == 'p')
+	    && root->vbox.active && root->vbox.w > 0. && root->vbox.h > 0.)
+	handle_subelement = TRUE;
+    else if (!id && root->w.length != -1 && root->h.length != -1)
+	handle_subelement = FALSE;
+
+    if (handle_subelement == TRUE) {
+	target = cairo_image_surface_create (CAIRO_FORMAT_RGB24,
+					 1, 1);
+	cr = cairo_create  (target);
+
+	draw = rsvg_cairo_new_drawing_ctx (cr, handle);
+    
+	if (!draw) {
+	    cairo_destroy (cr);
+	    cairo_surface_destroy (target);
+	    
+	    return FALSE;
+	}
+
+	while (sself != NULL) {
+	    draw->drawsub_stack = g_slist_prepend (draw->drawsub_stack, sself);
+	    sself = sself->parent;
+	}
+
+	rsvg_state_push (draw);
+	cairo_save (cr);
+
+	rsvg_node_draw ((RsvgNode *) handle->priv->treebase, draw, 0);
+	render = (RsvgCairoRender *) draw->render;
+
+	bbox.x = render->bbox.x;
+	bbox.y = render->bbox.y;
+	bbox.w = render->bbox.w;
+	bbox.h = render->bbox.h;
+
+	cairo_restore (cr);
+	rsvg_state_pop (draw);
+	rsvg_drawing_ctx_free (draw);
+	cairo_destroy (cr);
+	cairo_surface_destroy (target);
+
+	dimension_data->width = (int) (_rsvg_css_hand_normalize_length_sub (&root->w, bbox.w, handle->priv->dpi_x,
+									    bbox.w + bbox.x * 2, 12) + 0.5);
+	dimension_data->height = (int) (_rsvg_css_hand_normalize_length_sub (&root->h, bbox.h, handle->priv->dpi_y, 
+									    bbox.h + bbox.y * 2,
+									    12) + 0.5);
+    } else {
+	bbox.w = root->vbox.w; 
+	bbox.h = root->vbox.h; 
+
+	dimension_data->width = (int) (_rsvg_css_hand_normalize_length (&root->w, handle->priv->dpi_x,
+									bbox.w + bbox.x * 2, 12) + 0.5);
+	dimension_data->height = (int) (_rsvg_css_hand_normalize_length (&root->h, handle->priv->dpi_y, 
+									bbox.h + bbox.y * 2,
+									12) + 0.5);
     }
-
-    dimension_data->width = (int) (_rsvg_css_hand_normalize_length (&sself->w, handle->priv->dpi_x,
-                                                                    bbox.w + bbox.x * 2, 12) + 0.5);
-    dimension_data->height = (int) (_rsvg_css_hand_normalize_length (&sself->h, handle->priv->dpi_y,
-                                                                     bbox.h + bbox.y * 2,
-                                                                     12) + 0.5);
-
+    
     dimension_data->em = dimension_data->width;
     dimension_data->ex = dimension_data->height;
 
     if (handle->priv->size_func)
         (*handle->priv->size_func) (&dimension_data->width, &dimension_data->height,
                                     handle->priv->user_data);
+
+    return TRUE;
+}
+
+/**
+ * rsvg_handle_get_position_sub
+ * @handle: A #RsvgHandle
+ * @position_data: A place to store the SVG fragment's position.
+ * @id: An element's id within the SVG.
+ * For example, if you have a layer called "layer1" for that you want to get
+ * the position, pass "#layer1" as the id.
+ *
+ * Get the position of a subelement of the SVG file. Do not call from within
+ * the size_func callback, because an infinite loop will occur.
+ *
+ * Since: 2.22
+ */
+gboolean
+rsvg_handle_get_position_sub (RsvgHandle * handle, RsvgPositionData * position_data, const char *id)
+{
+	RsvgDrawingCtx		*draw;
+	RsvgNodeSvg			*root;
+	RsvgNode			*node;
+	RsvgBbox			 bbox;
+	RsvgDimensionData    dimension_data;
+	cairo_surface_t		*target = NULL;
+	cairo_t				*cr = NULL;
+	gboolean			 ret = FALSE;
+
+	g_return_val_if_fail (handle, FALSE);
+	g_return_val_if_fail (position_data, FALSE);
+
+	/* Short-cut when no id is given. */
+	if (NULL == id || '\0' == *id) {
+		position_data->x = 0;
+		position_data->y = 0;
+		return TRUE;
+	}
+
+	memset (position_data, 0, sizeof (*position_data));
+	memset (&dimension_data, 0, sizeof (dimension_data));
+
+	node = (RsvgNode *) rsvg_defs_lookup (handle->priv->defs, id);
+	if (!node) {
+		return FALSE;
+	} else if (node == (RsvgNode *) handle->priv->treebase) {
+		/* Root node. */
+		position_data->x = 0;
+		position_data->y = 0;
+		return TRUE;
+	}
+
+	root = (RsvgNodeSvg *) handle->priv->treebase;
+	if (!root)
+		return FALSE;
+
+	target = cairo_image_surface_create (CAIRO_FORMAT_RGB24, 1, 1);
+	cr = cairo_create  (target);
+	draw = rsvg_cairo_new_drawing_ctx (cr, handle);
+	if (!draw)
+	    goto bail;
+
+	while (node != NULL) {
+	    draw->drawsub_stack = g_slist_prepend (draw->drawsub_stack, node);
+	    node = node->parent;
+	}
+
+	rsvg_state_push (draw);
+	cairo_save (cr);
+
+	rsvg_node_draw ((RsvgNode *) handle->priv->treebase, draw, 0);
+	bbox.x = ((RsvgCairoRender *) draw->render)->bbox.x;
+	bbox.y = ((RsvgCairoRender *) draw->render)->bbox.y;
+	bbox.w = ((RsvgCairoRender *) draw->render)->bbox.w;
+	bbox.h = ((RsvgCairoRender *) draw->render)->bbox.h;
+
+	cairo_restore (cr);
+	rsvg_state_pop (draw);
+	rsvg_drawing_ctx_free (draw);
+
+	/* Calculate position.
+	 * TODO: figure out what "bbox.w + bbox.x * 2" is or means
+	 * (copied from rsvg_handle_get_dimensions_sub(). - robsta */
+	position_data->x = (int) (_rsvg_css_hand_normalize_length_sub (&root->x, bbox.x, handle->priv->dpi_x,
+									    bbox.w + bbox.x * 2, 12) + 0.5);
+	position_data->y = (int) (_rsvg_css_hand_normalize_length_sub (&root->y, bbox.y, handle->priv->dpi_y, 
+									    bbox.h + bbox.y * 2, 12) + 0.5);
+
+	/* Also calculate dimension for the size callback, while re're at it. */
+	dimension_data.width = (int) (_rsvg_css_hand_normalize_length_sub (&root->w, bbox.w, handle->priv->dpi_x,
+									    bbox.w + bbox.x * 2, 12) + 0.5);
+	dimension_data.height = (int) (_rsvg_css_hand_normalize_length_sub (&root->h, bbox.h, handle->priv->dpi_y, 
+									    bbox.h + bbox.y * 2, 12) + 0.5);
+
+	dimension_data.em = dimension_data.width;
+	dimension_data.ex = dimension_data.height;
+
+	if (handle->priv->size_func)
+	    (*handle->priv->size_func) (&dimension_data.width, &dimension_data.height,
+	                                handle->priv->user_data);
+
+        ret = TRUE;
+
+bail:
+	if (cr)
+		cairo_destroy (cr);
+	if (target)
+		cairo_surface_destroy (target);
+
+	return ret;
+}
+
+/** 
+ * rsvg_handle_has_sub:
+ * @handle: a #RsvgHandle
+ * @id: an element's id within the SVG
+ *
+ * Checks whether the element @id exists in the SVG document.
+ *
+ * Returns: %TRUE if @id exists in the SVG document
+ *
+ * Since: 2.22
+ */
+gboolean
+rsvg_handle_has_sub (RsvgHandle * handle,
+                     const char *id)
+{
+    g_return_val_if_fail (handle, FALSE);
+
+    if (G_UNLIKELY (!id || !id[0]))
+      return FALSE;
+
+    return rsvg_defs_lookup (handle->priv->defs, id) != NULL;
 }
 
 /** 
@@ -1610,10 +1772,6 @@ rsvg_handle_close (RsvgHandle * handle, GError ** error)
     return rsvg_handle_close_impl (handle, error);
 }
 
-#ifdef HAVE_GNOME_VFS
-#include <libgnomevfs/gnome-vfs.h>
-#endif
-
 /**
  * rsvg_init:
  *
