@@ -26,7 +26,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <glib/gslist.h>
+#include <glib.h>
 #include <math.h>
 #include <string.h>
 
@@ -107,7 +107,7 @@ static void rsvg_cairo_transformed_image_bounding_box (
     *y1 = ceil (t > y11 ? t : y11);
 }
 
-static RsvgDrawingCtx *
+RsvgDrawingCtx *
 rsvg_cairo_new_drawing_ctx (cairo_t * cr, RsvgHandle * handle)
 {
     RsvgDimensionData data;
@@ -151,6 +151,7 @@ rsvg_cairo_new_drawing_ctx (cairo_t * cr, RsvgHandle * handle)
     draw->vb.h = data.ex;
     draw->pango_context = NULL;
     draw->drawsub_stack = NULL;
+    draw->ptrs = NULL;
 
     rsvg_state_push (draw);
     state = rsvg_state_current (draw);
