@@ -32,7 +32,7 @@
 #include <libxml/SAX.h>
 #include <libxml/xmlmemory.h>
 #include <pango/pango.h>
-#include <glib/gslist.h>
+#include <glib.h>
 #include <glib-object.h>
 #include <math.h>
 
@@ -125,6 +125,7 @@ struct RsvgSaxHandler {
 
 struct RsvgHandlePrivate {
     gboolean is_disposed;
+    gboolean is_closed;
 
     RsvgSizeFunc size_func;
     gpointer user_data;
@@ -166,6 +167,7 @@ struct RsvgHandlePrivate {
     gboolean first_write;
     gboolean is_gzipped;
     void *gzipped_data;         /* really a GsfOutput */
+    gboolean in_loop;		/* see get_dimension() */
 };
 
 typedef struct {
@@ -181,12 +183,12 @@ struct RsvgDrawingCtx {
     GError **error;
     RsvgDefs *defs;
     gchar *base_uri;
-    GMemChunk *state_allocator;
     PangoContext *pango_context;
     double dpi_x, dpi_y;
     RsvgViewBox vb;
     GSList *vb_stack;
     GSList *drawsub_stack;
+    GSList *ptrs;
 };
 
 /*Abstract base class for context for our backends (one as yet)*/
@@ -344,6 +346,9 @@ void rsvg_bbox_clip	(RsvgBbox * dst, RsvgBbox * src);
 double _rsvg_css_normalize_length	(const RsvgLength * in, RsvgDrawingCtx * ctx, char dir);
 double _rsvg_css_hand_normalize_length	(const RsvgLength * in, gdouble pixels_per_inch,
 					 gdouble width_or_height, gdouble font_size);
+double _rsvg_css_hand_normalize_length_sub	(const RsvgLength * in, gdouble length,
+						 gdouble pixels_per_inch, gdouble width_or_height, 
+						 gdouble font_size);
 
 RsvgLength _rsvg_css_parse_length (const char *str);
 
@@ -360,16 +365,16 @@ void rsvg_return_if_fail_warning (const char *pretty_function,
 #define rsvg_return_if_fail(expr, error)		G_STMT_START{			\
      if G_LIKELY(expr) { } else       					\
        {								\
-	 rsvg_return_if_fail_warning (G_GNUC_PRETTY_FUNCTION,		        \
-		                   #expr, error);				\
+	 rsvg_return_if_fail_warning (G_STRFUNC,			\
+		                   #expr, error);			\
 	 return;							\
        };				}G_STMT_END
 
 #define rsvg_return_val_if_fail(expr,val,error)	G_STMT_START{			\
      if G_LIKELY(expr) { } else						\
        {								\
-	 rsvg_return_if_fail_warning (G_GNUC_PRETTY_FUNCTION,		        \
-		                   #expr, error);				\
+	 rsvg_return_if_fail_warning (G_STRFUNC,	    		\
+		                   #expr, error);			\
 	 return (val);							\
        };				}G_STMT_END
 
